function [L, H, w] = genGrids(M, n, v)
% M: number of bands;   n: grid size;   v: wind speed
% L: length scales;   H: spectra;   w: angular wave frequencies

  g = 9.81;                   % gravitational acceleration
  w0 = 0.877*g/norm(v);       % angular peak frequency
  k1 = 20/norm(v)^2;          % maximum wave number in the lowest grid
  kM = 200.0;                 % maximum wave number in the highest grid

  L = cell(M, 1);             % length scales
  H = cell(M, 1);             % amplitudes and phases
  w = cell(M, 1);             % angular frequencies

  for m=1:M
    if M==1                   % single band approach
      L{m} = pi*n/k1;
    else                      % multi-band approach
      L{m} = pi*n*(k1/kM)^((m-1)/(M-1))/k1;
    end

    [kx ky] = meshgrid(0:n-1, 0:n-1);                 % wave vectors
    kx = 2.0*pi*(kx-n*(kx>=n/2))/L{m};
    ky = 2.0*pi*(ky-n*(ky>=n/2))/L{m};
    k = sqrt(kx.^2+ky.^2);                            % wave numbers
    w{m} = sqrt(g*k);                                 % Eq. (4)

    S = 0.0081*g^2*exp(-1.25*(w0./w{m}).^4)./w{m}.^5; % Eq. (1)
    s = 16.0*(w{m}/w0).^((w{m}<w0)*7.5-2.5);          % Mitsuyasu model
    theta = acos((v(1)*kx+v(2)*ky)./(norm(v)*k));
    D = gamma(s+1.0).*(cos(theta/2.0)).^(2.0*s)...
        ./(2.0*sqrt(pi)*gamma(s+0.5));                % Eq. (2)
    Psi = S.*D.*sqrt(g./k.^3)/2.0;                    % Eq. (5)
    Psi(isnan(Psi)) = 0.0;

    a = sqrt(-2.0*Psi.*log(rand(n)))*2.0*pi/L{m};     % Eq. (7)
    phi = 2.0*pi*rand(n);                             % random phases
    
    if (m>1)    % remove contributions represented in the lower bands
      a(abs(kx)<n*pi/L{m-1} & abs(ky)<n*pi/L{m-1}) = 0.0;
    end

    H{m} = a.*exp(i*phi);                       % Fourier coefficients
  end
end
